@@ -3,7 +3,11 @@
 from __future__ import absolute_import, unicode_literals
 
 import sys
+import os
 import warnings
+import glob
+from importlib import import_module
+
 
 import ruamel.yaml
 from ruamel.yaml.error import *                                # NOQA
@@ -40,8 +44,8 @@ enforce = object()
 # subset of abbreviations, which should all caps according to PEP8
 
 class YAML(object):
-    def __init__(self, _kw=enforce, typ=None, pure=False):
-        # type: (Any, Any, Any) -> None
+    def __init__(self, _kw=enforce, typ=None, pure=False, plug_ins=None):
+        # type: (Any, Any, Any, Any) -> None
         """
         _kw: not used, forces keyword arguments in 2.7 (in 3 you can do (*, safe_load=..)
         typ: 'rt'/None -> RoundTripLoader/RoundTripDumper,  (default)
@@ -49,13 +53,18 @@ class YAML(object):
              'unsafe'  -> normal/unsafe Loader/Dumper
              'base'    -> baseloader
         pure: if True only use Python modules
+        plug_ins: a list of plug-in files
         """
         if _kw is not enforce:
             raise TypeError("{}.__init__() takes no positional argument but at least "
                             "one was given ({!r})".format(self.__class__.__name__, _kw))
 
         self.typ = 'rt' if typ is None else typ
-        self.Resolver = ruamel.yaml.resolver.VersionedResolver
+        self.plug_ins = []  # type: List[Any]
+        for pu in ([] if plug_ins is None else plug_ins) + self.official_plug_ins():
+            file_name = pu.replace('/', '.')
+            self.plug_ins.append(import_module(file_name))
+        self.Resolver = ruamel.yaml.resolver.VersionedResolver               # type: Any
         self.allow_unicode = True
         self.Reader = None       # type: Any
         self.Scanner = None      # type: Any
@@ -89,6 +98,14 @@ class YAML(object):
             self.Parser = ruamel.yaml.parser.Parser if pure or CParser is None else CParser
             self.Composer = ruamel.yaml.composer.Composer
             self.Constructor = ruamel.yaml.constructor.Constructor
+        else:
+            for module in self.plug_ins:
+                if getattr(module, 'typ', None) == self.typ:
+                    module.init_typ(self)
+                    break
+            else:
+                raise NotImplementedError(
+                    'typ "{}"not recognised (need to install plug-in?)'.format(self.typ))
         self.stream = None
         self.canonical = None
         self.indent = None
@@ -106,6 +123,7 @@ class YAML(object):
         self.tags = None
         self.default_style = None
         self.default_flow_style = None
+        self.top_level_block_style_scalar_no_indent_error_1_1 = False
 
     @property
     def reader(self):
@@ -139,7 +157,7 @@ class YAML(object):
                     #     # pathlib.Path() instance
                     #     setattr(self, attr, CParser(self._stream))
                     # else:
-                    setattr(self, attr, CParser(self._stream))  # type: ignore
+                    setattr(self, attr, CParser(self._stream))
                     # self._parser = self._composer = self
                     # print('scanner', self.loader.scanner)
 
@@ -233,6 +251,14 @@ class YAML(object):
             return constructor.get_single_data()
         finally:
             parser.dispose()
+            try:
+                self._reader.reset_reader()  # type: ignore
+            except AttributeError:
+                pass
+            try:
+                self._scanner.reset_scanner()  # type: ignore
+            except AttributeError:
+                pass
 
     def load_all(self, stream, _kw=enforce):  # , skip=None):
         # type: (StreamTextType, Any) -> Any
@@ -253,6 +279,14 @@ class YAML(object):
                 yield constructor.get_data()
         finally:
             parser.dispose()
+            try:
+                self._reader.reset_reader()  # type: ignore
+            except AttributeError:
+                pass
+            try:
+                self._scanner.reset_scanner()  # type: ignore
+            except AttributeError:
+                pass
 
     def get_constructor_parser(self, stream):
         # type: (StreamTextType) -> Any
@@ -298,32 +332,34 @@ class YAML(object):
                 return loader, loader
         return self.constructor, self.parser
 
-    def dump(self, data, stream=None):
-        # type: (Any, StreamType) -> Any
-        return self.dump_all([data], stream)
+    def dump(self, data, stream, _kw=enforce, transform=None):
+        # type: (Any, StreamType, Any, Any) -> Any
+        return self.dump_all([data], stream, _kw, transform=transform)
 
-    def dump_all(self, documents, stream=None):
-        # type: (Any, StreamType) -> Any
+    def dump_all(self, documents, stream, _kw=enforce, transform=None):
+        # type: (Any, StreamType, Any, Any) -> Any
         """
         Serialize a sequence of Python objects into a YAML stream.
         If stream is None, return the produced string instead.
         """
-        # The stream should have the methods `write` and possibly `flush`.
         if not hasattr(stream, 'write') and hasattr(stream, 'open'):
             # pathlib.Path() instance
             with stream.open('w') as fp:   # type: ignore
-                return self.dump_all(documents, fp)
-        getvalue = None
+                return self.dump_all(documents, fp, _kw, transform=transform)
+        if _kw is not enforce:
+            raise TypeError("{}.dump(_all) takes two positional argument but at least "
+                            "three were given ({!r})".format(self.__class__.__name__, _kw))
+        # The stream should have the methods `write` and possibly `flush`.
         if self.top_level_colon_align is True:
             tlca = max([len(str(x)) for x in documents[0]])  # type: Any
         else:
             tlca = self.top_level_colon_align
-        if stream is None:
+        if transform is not None:
+            fstream = stream
             if self.encoding is None:
                 stream = StringIO()
             else:
                 stream = BytesIO()
-            getvalue = stream.getvalue
         serializer, representer, emitter = \
             self.get_serializer_representer_emitter(stream, tlca)
         try:
@@ -343,8 +379,8 @@ class YAML(object):
                 # self.dumper.dispose()  # cyaml
             delattr(self, "_serializer")
             delattr(self, "_emitter")
-        if getvalue is not None:
-            return getvalue()
+        if transform:
+            fstream.write(transform(stream.getvalue()))   # type: ignore
         return None
 
     def get_serializer_representer_emitter(self, stream, tlca):
@@ -383,9 +419,9 @@ class YAML(object):
                                   explicit_end=explicit_end,
                                   version=version, tags=tags)
                 selfx._emitter = selfx._serializer = selfx._representer = selfx
-                Representer.__init__(selfx, default_style=default_style,
-                                     default_flow_style=default_flow_style)
-                Resolver.__init__(selfx)
+                self.Representer.__init__(selfx, default_style=default_style,
+                                          default_flow_style=default_flow_style)
+                rslvr.__init__(selfx)
         self._stream = stream
         dumper = XDumper(stream)
         self._emitter = self._serializer = dumper
@@ -408,6 +444,14 @@ class YAML(object):
         else:
             return list(*args)
 
+    # helpers
+    def official_plug_ins(self):
+        # type: () -> Any
+        bd = os.path.dirname(__file__)
+        gpbd = os.path.dirname(os.path.dirname(bd))
+        res = [x.replace(gpbd, '')[1:-3] for x in glob.glob(bd + '/*/__plug_in__.py')]
+        return res
+
 
 ########################################################################################
 
@@ -475,9 +519,9 @@ def load(stream, Loader=None, version=None, preserve_quotes=None):
         Loader = UnsafeLoader
     loader = Loader(stream, version, preserve_quotes=preserve_quotes)
     try:
-        return loader._constructor.get_single_data()
+        return loader._constructor.get_single_data()  # type: ignore
     finally:
-        loader._parser.dispose()
+        loader._parser.dispose()  # type: ignore
 
 
 def load_all(stream, Loader=None, version=None, preserve_quotes=None):
@@ -491,10 +535,10 @@ def load_all(stream, Loader=None, version=None, preserve_quotes=None):
         Loader = UnsafeLoader
     loader = Loader(stream, version, preserve_quotes=preserve_quotes)
     try:
-        while loader._constructor.check_data():
-            yield loader._constructor.get_data()
+        while loader._constructor.check_data():  # type: ignore
+            yield loader._constructor.get_data()  # type: ignore
     finally:
-        loader._parser.dispose()
+        loader._parser.dispose()  # type: ignore
 
 
 def safe_load(stream, version=None):
